#!/usr/bin/perl -w
# vim: set ts=4 sw=4 et : */

use strict;
use Glib qw(FALSE TRUE);
use Gtk2 -init;
use Gtk2::Cairo;
use Cairo;

use constant ANIMATE_ROTATE_STEP => 0.05;
use constant ANIMATE_FRAME_DELAY => 40;

use constant ROTATE_MAX => 6.28;  # 2*pi

use constant MARGIN => 20;
use constant INITIAL_SIZE => 200;

my $timerid = 0;

sub paint {
    my ($widget, $cairo, $range) = @_;

    my $width  = $widget->allocation->width;
    my $height = $widget->allocation->height;
    my $box_size = ($width+$height)/6;

    $cairo->save;

    $cairo->default_matrix;
    $cairo->translate ($width/2, $height/2);

    $cairo->rotate ($range->get_value);
    $cairo->rectangle (-$box_size, -$box_size, $box_size, $box_size);
    $cairo->set_rgb_color (1, 0, 0);
    $cairo->fill;
    $cairo->restore;
}

sub slider_changed {
    my ($range, $gtkcairo) = @_;
    $gtkcairo->queue_draw;
}

sub animate_step {
    my ($range) = @_;
    my $newval = $range->get_value + ANIMATE_ROTATE_STEP;
    $newval -= ROTATE_MAX if $newval > ROTATE_MAX;
    $range->set_value ($newval);
    return TRUE;
}

sub animate_toggled {
    my ($tb, $range) = @_;
    my $active = $tb->get_active;
    if ($active && !$timerid) {
        $timerid = Glib::Timeout->add (ANIMATE_FRAME_DELAY,
                                       \&animate_step, $range);
    } elsif (!$active && $timerid) {
        Glib::Source->remove ($timerid);
        $timerid = 0;
    }
}

{
    my $win = Gtk2::Window->new;
    $win->set_title ("GtkCairo Demo");
    $win->signal_connect (delete_event => sub { Gtk2->main_quit; FALSE });

    my $vbox = Gtk2::VBox->new (FALSE, 6);
    $vbox->set_border_width (12);

    my $frame = Gtk2::Frame->new;
    $frame->set_shadow_type ('in');

    my $slider = Gtk2::HScale->new_with_range (0, ROTATE_MAX, 0.05);

    my $gtkcairo = Gtk2::Cairo->new;
    $gtkcairo->set_size_request (INITIAL_SIZE, INITIAL_SIZE);
    $gtkcairo->signal_connect (paint => \&paint, $slider);

    $frame->add ($gtkcairo);
    $vbox->pack_start ($frame, TRUE, TRUE, 0);

    $slider->set_draw_value (FALSE);
    $slider->signal_connect (value_changed => \&slider_changed, $gtkcairo);
    $vbox->pack_start ($slider, FALSE, FALSE, 0);

    my $animate = Gtk2::CheckButton->new_with_label ("Animate");
    $animate->signal_connect (toggled => \&animate_toggled, $slider);
    $vbox->pack_start ($animate, FALSE, FALSE, 0);

    $win->add ($vbox);

    $win->show_all;

    Gtk2->main;
}

